{% load ddrr %}
{% if formatter.colors %}{{ 'Request:'|colorize:"green" }}{% else %}Request:{% endif %} {{ ddrr.method }} {{ ddrr.path }}{{ ddrr.query_string }}
{% for header, value in ddrr.headers.items %}{{ header }}: {{ value }}
{% endfor %}{% if ddrr.body %}
{{ ddrr.body }}{% endif %}🔚
